//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 22/06/02; 21:27:28

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "graphic.h"
#include "mstrmind.h"



//--------------------------------------------------------------------------

char * Plane0;
TGame * Game;

char StatusMsg[32];
char KeyPegsStr[8];

//--------------------------------------------------------------------------

void GetCombRW(TComb * Comb, TComb * Solution)
{
	int a,b;
	
	int try_used[4];
	int sol_used[4];
	
	Comb->KeyPegs_R = 0;
	Comb->KeyPegs_W = 0;
	
	for (a=0;a<4;a++)
	{
		try_used[a] = 0;
		sol_used[a] = 0;
	}
	
	
	
	for (a=0;a<4;a++)
	if (!try_used[a] && !sol_used[a])
	if (Comb->Items[a]==Solution->Items[a])
	{
		Comb->KeyPegs_R++;
		try_used[a] = 1;
		sol_used[a] = 1;
	}
	
	for (a=0;a<4;a++)
	for (b=0;b<4;b++)
	if (!try_used[a] && !sol_used[b])
	if (Comb->Items[a]==Solution->Items[b])
	{
		Comb->KeyPegs_W++;
		try_used[a] = 1;
		sol_used[b] = 1;
	}
	
	
	
}

//--------------------------------------------------------------------------

void NewGame(void)
{
	int a;
	
	strcpy(KeyPegsStr,"ABCDEFGH");
	strcpy(StatusMsg,"");
	
	Game->CodeTry = 0;
	Game->CodePegNum = 0;
	Game->show_solution = 0;
	
	// COMB. 0
	
	Game->Combs[0].Items[0] = 0;
	Game->Combs[0].Items[1] = 0;
	Game->Combs[0].Items[2] = 0;
	Game->Combs[0].Items[3] = 0;
	
	Game->Combs[0].KeyPegs_R=0;
	Game->Combs[0].KeyPegs_W=0;
	
	// SOLUTION
	for (a=0;a<4;a++)
	Game->Solution.Items[a] = random(8);
	
}
//--------------------------------------------------------------------------

void RefreshScreen(void)
{
	int a;
	char buffer[50];
	
	PortSet(Plane0,239,127);
	ClrScr();
	
	
	for (a=0;a<=Game->CodeTry;a++)
	ShowComb(12*a,0,&(Game->Combs[a]));
	
	if (Game->show_solution)
	ShowComb(SOLUTION_LEFT,0,&(Game->Solution));
	
	
	sprintf(buffer,"Try : %d/%d",Game->CodeTry+1,TRIES_MAX);
	DrawStr(0,MSG_TOP,buffer,A_NORMAL);
	
	DrawStr(80,MSG_TOP,StatusMsg,A_NORMAL);
	
	PortRestore();
	FastCopyScreen(Plane0,LCD_MEM);
	
}
//--------------------------------------------------------------------------

void ShowComb(int x, int y, TComb * Comb)
{
	int a;
	
	// CODEPEGS
	
	for (a=0;a<4;a++)
	{
		Sprite16_OR(x,y+12*a,16,CodePeg_C0,Plane0);
		DrawChar(x+3,y+12*a+2,KeyPegsStr[Comb->Items[a]],A_NORMAL);
	}
	
	// KEYPEGS
	
	if (Comb==&Game->Solution)
	return;
	
	unsigned char * src;
	
	for (a=0;a<4;a++)
	{
		if ((Comb->KeyPegs_R+Comb->KeyPegs_W)<(a+1))
		break;
		
		src = (Comb->KeyPegs_R>=(a+1))?KeyPeg_R:KeyPeg_W;
		Sprite8_OR(x+2,KEYPEGS_TOP+(8*a),8,src,Plane0);
	}
	
}
//--------------------------------------------------------------------------

void GetPossibleSolution(void)
{
	int a,b;
	
	TComb CombA;
	TComb*CombPossibleSolution=&CombA;
	
	TComb CombTest;
	
	for (;;)
	{
		retry:
		
		for (a=0;a<4;a++)
		CombPossibleSolution->Items[a] = random(8);
		
		// IF FIRST TRY -> OK
		if (Game->CodeTry==0)
		break;
		
		// DIFFERENT COMBINATION
		
		for (a=0;a<Game->CodeTry;a++)
		{
			
			int Count = 0;
			for (b=0;b<4;b++)
			if (Game->Combs[a].Items[b]==CombPossibleSolution->Items[b])
			Count++;
			
			if (Count==4)
			goto retry;
		}
		
		// SUITS TO KEYPEGS
		
		for (a=0;a<Game->CodeTry;a++)
		{
			CombTest=Game->Combs[a];
			GetCombRW(&CombTest,CombPossibleSolution);
			
			if (!(Game->Combs[a].KeyPegs_R==CombTest.KeyPegs_R
			&&    Game->Combs[a].KeyPegs_W==CombTest.KeyPegs_W))
			goto retry;
			
		}
		
		break;
		
	}
	
	CombPossibleSolution->KeyPegs_R=0;
	CombPossibleSolution->KeyPegs_W=0;
	Game->Combs[Game->CodeTry]=*CombPossibleSolution;
	
}
//--------------------------------------------------------------------------


// Main Function
void _main(void)
{
	int a;
	short key;
	
	TGame GameA;
	Game=&GameA;
	
	
	if (!(Plane0=malloc(LCD_SIZE)))
	goto end_prog;
	
	randomize();
	
	
	
	FontSetSys(0);
	clrscr();
	printf("MasterMind 1.2 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	NewGame();
	RefreshScreen();
	
	for (;;)
	{
		
		
		
		TComb*Comb=&Game->Combs[Game->CodeTry];
		
		Sprite16_XOR(12*Game->CodeTry,12*Game->CodePegNum,16,CodePeg_SL,LCD_MEM);
		key = ngetchx();
		Sprite16_XOR(12*Game->CodeTry,12*Game->CodePegNum,16,CodePeg_SL,LCD_MEM);
		
		if (key==KEY_UP)
		{
			Game->CodePegNum--;
			Game->CodePegNum&=3;
		}
		
		else if (key==KEY_DOWN)
		{
			Game->CodePegNum++;
			Game->CodePegNum&=3;
		}
		
		else if (key==KEY_LEFT)
		{
			Comb->Items[Game->CodePegNum]--;
			Comb->Items[Game->CodePegNum]&=7;
			RefreshScreen();
		}
		
		else if (key==KEY_RIGHT)
		{
			Comb->Items[Game->CodePegNum]++;
			Comb->Items[Game->CodePegNum]&=7;
			RefreshScreen();
		}
		
		else if (key==KEY_ENTER)
		{
			GetCombRW(Comb,&Game->Solution);
			RefreshScreen();
			
			
			if (Game->Combs[Game->CodeTry].KeyPegs_R==4)
			{
				strcpy(StatusMsg,"You win!");
				Game->show_solution = 1;
				RefreshScreen();
				ngetchx();
				
				NewGame();
				RefreshScreen();
			}
			else 
			{
				
				if (Game->CodeTry==(TRIES_MAX-1))
				{
					strcpy(StatusMsg,"Game over!");
					Game->show_solution = 1;
					RefreshScreen();
					ngetchx();
					
					NewGame();
					RefreshScreen();
				}
				else
				{
					Game->CodeTry++;
					
					for (a=0;a<4;a++)
					Game->Combs[Game->CodeTry].Items[a] = Game->Combs[Game->CodeTry-1].Items[a];
					
					Game->Combs[Game->CodeTry].KeyPegs_R=0;
					Game->Combs[Game->CodeTry].KeyPegs_W=0;
					
					RefreshScreen();
				}
				
			}
			
		}
		
		else if (key==KEY_APPS )
		{
			off();
		}
		
		else if (key==KEY_F1 )
		{
			strcpy(StatusMsg,"Thinking...");
			RefreshScreen();
			GetPossibleSolution();
			strcpy(StatusMsg,"");
			RefreshScreen();
		}
		
		else if (key==KEY_F2 )
		{
			NewGame();
			RefreshScreen();
		}
		
		else if (key==KEY_ESC)
		{
			
			clrscr();
			printf("[ESC]   : Continue \n");
			printf("[ENTER] : Exit \n");
			
			while ((key = ngetchx())!=KEY_ENTER && key!=KEY_ESC);
			
			if (key==KEY_ENTER)
			break;
			
			RefreshScreen();
			
		}
		
	}
	
	end_prog:
	
	if (Plane0)
	free(Plane0);
	
}



